@@ -1,6 +1,6 @@
 /*
  * Mausezahn - A fast versatile traffic generator
- * Copyright (C) 2008 Herbert Haas
+ * Copyright (C) 2008, 2009 Herbert Haas
  * 
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License version 2 as published by the 
@@ -69,8 +69,8 @@ int rcv_rtp_init()
 		"| Note:\n"
 		"|\n"
 		"|  Mausezahn can log actual realtime measurement data in data files (in the specified path or\n"
-		"|  current directory) but always prints the moving average on the commandline (can be disabled using the\n"
-		"|  'quiet' option (-q)).\n" 
+		"|  current directory) but always prints the moving average on the command line (this can be disabled\n"
+		"|  using the 'quiet' option (-q)).\n" 
 		"|\n"
 		"|  The realtime data file(s) consist of three columns:\n"
 		"|\n"
@@ -92,17 +92,22 @@ int rcv_rtp_init()
 		"|  The delta-RX values in the third column can be used to analyze the inter-packet delay observed\n"
 		"|  at the local station.\n"
 		"|  \n"
-		"|  The average data files consist of seven columns:\n"
+		"|  The average data files consist of ten columns:\n"
 		"|  \n"
 		"|  1. relative timestamp in seconds\n" 
 		"|  2. minimum jitter\n"
 		"|  3. average jitter\n"
 		"|  4. minimum jitter\n"
-		"|  5. minimum interval\n"
-		"|  6. average interval\n"
-		"|  7. maximum interval\n"
+		"|  5. estimated jitter according RFC-3550\n"
+		"|  6. min_deltaRX\n"
+		"|  7. avg_deltaRX\n"
+		"|  8. max_deltaRX\n"
+		"|  9. packet drop count (total)\n"
+		"|  10. packet disorder count (total)\n"
 		"|  \n"
 		"|  All measurement values are in usec and refer to the current set of samples (see parameter 'num').\n"
+	        "|  Note that an RFC-conform jitter (smoothed mean deviation) is calculated and collected in column\n"
+	        "|  five.\n"
 		"|  \n"
 		"| EXAMPLE USAGE:\n"
 		"|\n"		
@@ -112,7 +117,7 @@ int rcv_rtp_init()
 		"|\n"
 		"|  At the RX-station (10.3.3.42) enter:\n"
 		"|\n"
-		"|    # mz eth0 -T rtp \"logg, path=/tmp/mz/\"\n"
+		"|    # mz eth0 -T rtp \"log, path=/tmp/mz/\"\n"
 		"|\n"
 		"\n", TIME_COUNT_MAX, TIME_COUNT);
 	exit(0);
@@ -136,11 +141,15 @@ int rcv_rtp_init()
    if (getarg(tx.arg_string,"path", argval)==1)
      {
 	len = strlen(argval);
+	if (len>128) {     
+		fprintf(stderr, " mz/Error: path must not exceed 128 characters!\n");
+		exit (-1);
+	}
 	if (argval[len-1]!='/')
 	  {
 	     strncat(argval, "/",1); // ensure that all paths end with "/"
 	  }
-	strncpy(path, argval, 256);
+	strncpy(path, argval, 128);
      }
 
 
@@ -185,8 +194,8 @@ int rcv_rtp_init()
      {
 	// get a new filename
 	timestamp_human(filename, "rtp_avg_");
-	strncpy(dummy, path, 256);
-	strncat(dummy, filename, 256);
+	strncpy(dummy, path, 128);
+	strncat(dummy, filename, 64);
 	if (verbose) fprintf(stderr, " mz: Will open %s\n", dummy);
 	
 	fp = fopen (dummy, "w+");
@@ -205,19 +214,20 @@ int rcv_rtp_init()
 		    "#  2. min_jitter\n"
 		    "#  3. avg_jitter\n"
 		    "#  4. max_jitter\n"
-		    "#  5. min_deltaRX\n"
-		    "#  6. avg_deltaRX\n"
-		    "#  7. max_deltaRX\n"
-		    "#  8. packet drop count (total)\n"
-		    "#  9. packet disorder count (total)\n");
+		    "#  5. estimated jitter according RFC-3550\n"
+		    "#  6. min_deltaRX\n"
+		    "#  7. avg_deltaRX\n"
+		    "#  8. max_deltaRX\n"
+		    "#  9. packet drop count (total)\n"
+		    "#  10. packet disorder count (total)\n");
 
 	
 	if (rtp_log==2) ///////////// also detailed log required /////////////
 	  {
 	     // get a new filename
 	     timestamp_human(filename, "rtp_rt_");
-	     strncpy(dummy, path, 256);
-	     strncat(dummy, filename, 256);
+	     strncpy(dummy, path, 128);
+	     strncat(dummy, filename, 64);
 	     if (verbose) fprintf(stderr, " mz: Will open %s\n", dummy);
 	     
 	     fp2 = fopen (dummy, "w+");
@@ -237,6 +247,7 @@ int rcv_rtp_init()
    
    drop=0;
    dis=0;
+   jitter_rfc=0;
    
    return 0;
 }
@@ -377,6 +388,8 @@ void got_rtp_packet(u_char *args,
    char dummy[256];
    unsigned char *dum;
    
+   long int
+     ltemp;
      
    
 
@@ -490,8 +503,13 @@ void got_rtp_packet(u_char *args,
 		  // (pseudo)jitter = deltaRX - deltaTX, hence we have positive and negative jitter (delay deviations)
 		  // jitter entries are in +/- microseconds
 		  jitter[i] = (deltaRX.tv_sec*1000000 + deltaRX.tv_usec) - (deltaTX.tv_sec*1000000 + deltaTX.tv_usec);
+		  // Calculate RFC 3550 jitter estimation. According to that RFC the jitter should be
+		  // measured in timestamp units; however currently Mausezahn uses microseconds.
+		  // AFAIK G.711 timestamp units would be 125 usec, so jitter/=125 would be sufficient(?)
+		  ltemp = labs(jitter[i]) - jitter_rfc;
+		  jitter_rfc += (ltemp>>4);  
 		  // Add previous pseudojitter to get the true jitter (See Documentation!)
-		  jitter[i] += jitter[i-1]; 
+		  jitter[i] += jitter[i-1];
 		  //
 		  ////////////////////////////////////////////////////////////////////////////////////////////////////
 		  
@@ -555,10 +573,12 @@ void got_rtp_packet(u_char *args,
 	       {  dum =  (unsigned char*) &ip->src;
 		  fprintf(stdout, 
 			  "Jitter (min/avg/max) = %lu/%lu/%lu usec; "
+			  "Jitter_RFC = %li; "
 			  "delta-RX (min/avg/max) = %lu/%lu/%lu usec (got %u packets from %u.%u.%u.%u, %u lost, %u out of order).\n",
 			  (long unsigned int) jitter_min,
 			  (long unsigned int) jitter_avg,
 			  (long unsigned int) jitter_max,
+			  jitter_rfc,
 			  (long unsigned int) deltaRX_min,
 			  (long unsigned int) deltaRX_avg,
 			  (long unsigned int) deltaRX_max,
@@ -581,11 +601,12 @@ void got_rtp_packet(u_char *args,
 	       {
 		  getcurtime(&curtv);
 		  fprintf(fp, 
-			  "%f, %lu, %lu, %lu, %lu, %lu, %lu, %u, %u\n",
+			  "%f, %lu, %lu, %lu, %li, %lu, %lu, %lu, %u, %u\n",
 			  (float) curtime/1000000,
 			  (long unsigned int) jitter_min,
 			  (long unsigned int) jitter_avg,
 			  (long unsigned int) jitter_max,
+			  jitter_rfc,
 			  (long unsigned int) deltaRX_min,
 			  (long unsigned int) deltaRX_avg,
 			  (long unsigned int) deltaRX_max,
@@ -611,8 +632,8 @@ void got_rtp_packet(u_char *args,
 		  if (verbose) fprintf(stderr, " mz: %s written.\n",filename);
 		  
 		  timestamp_human(filename, "rtp_");  // get a new filename
-		  strncpy(dummy, path, 256);
-		  strncat(dummy, filename, 256);
+		  strncpy(dummy, path, 128);
+		  strncat(dummy, filename, 64);
 		  
 		  if (verbose) fprintf(stderr, " mz: Will open %s\n", dummy);
 		  
